
#include "main.h"

// win1251 - LCD translation table ///////////
const unsigned char TransTable[] =
{0x41,0xA0,0x42,0xA1,0xE0,0x45,0xA3,0xA4,				//C0-C7		,,,,,,,
0xA5,0xA6,0x4B,0xA7,0x4D,0x48,0x4F,0xA8,				//C8-CF		,,,,,,,
0x50,0x43,0x54,0xA9,0xAA,0x58,0xE1,0xAB,				//D0-D7		,,,,,,,
0xAC,0xE2,0xAD,0xAE,0x62,0xAF,0xB0,0xB1,				//D8-DF		,,,,,,,
0x61,0xB2,0xB3,0xB4,0xE3,0x65,0xB6,0xB7,				//E0-E7		,,,,,,,
0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0x6F,0xBE,				//E8-EF		,,,,,,,
0x70,0x63,0xBF,0x79,0xE4,0x78,0xE5,0xC0,				//F0-F7		,,,,,,,
0xC1,0xE6,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7};				//F8-FF		,,,,,,,
///////////////////////////////////////////////

//---------------- LCD global variables section ------------------------
unsigned char lcd_x_cnt=0;
unsigned char lcd_y_cnt=0;
unsigned char _base_y[4]={0x80, 0xC0, 0x80+lcd_max_x, 0xC0+lcd_max_x}; /*HD44780 or KS0066 controller map*/


void lcd_init(void)
{
LCD_EN_CLR();
LCD_RW_CLR();
LCD_RS_CLR();

LCD_DAT_CLR();
LCD_DAT_IN();//Set HighZ

delay_ms(50);//wait for LCD internal initialization

lcd_init_write(0x30);
lcd_init_write(0x30);
lcd_init_write(0x30);
lcd_init_write(0x20);//set 4-bit mode
lcd_write_command(0x28);//4 bit mode, 2 strings
lcd_write_command(0x04);//AC=decrement, S=no shift screen
lcd_write_command(0x85);

lcd_clear();
}

/// Clear LCD ///
void lcd_clear(void)
{
        lcd_write_command(0x02);
        lcd_write_command(0x0C);
        lcd_write_command(0x01);
        lcd_gotoxy(0,0);
}

/// Print single char ///
void lcd_putchar(char chr)
{
if (chr==0xA8) chr=0xA2;							//
if (chr==0xB8) chr=0xB5;							//
if (chr > 0xBF) chr = TransTable [(chr-0xC0)];		//  
lcd_write_data(chr);
if(++lcd_x_cnt==lcd_max_x)
        {
        lcd_x_cnt=0;
        if(++lcd_y_cnt==lcd_max_y) lcd_y_cnt=0;
        lcd_gotoxy(0,lcd_y_cnt);
        }
}

/*--------------------------------------------------------
Print integer value on LCD
Arguments:
num - value to print
lead_zero - how many zeroes to place before value
use_space - place spaces on not printed zeroes(SPACE_ALLOW, SPACE_SUPRESS)
point_pos - decimal point position from right(DEC_POINT, DEC_NOPOINT) 
---------------------------------------------------------*/
void lcd_print_int
(unsigned int num, unsigned char lead_zero, char use_space, char dec_point)
{
        unsigned int temp;
        unsigned char c[7],i;
        
        temp=num;
        
        for(i=0;i<7;i++)
                c[i]='0';
        
        while(num>=1000000){num-=1000000; ++c[6];};
        while(num>=100000){num-=100000;   ++c[5];};
		while(num>=10000) {num-=10000;    ++c[4];};
        while(num>=1000)  {num-=1000;  	  ++c[3];};
        while(num>=100)   {num-=100;      ++c[2];};
        while(num>=10)    {num-=10;       ++c[1];};
        while(num)        {--num;         ++c[0];};
        
        if((temp>=1000000)||(lead_zero>6)) lcd_putchar(c[6]);
        else    if(use_space==SPACE_ALLOW) lcd_putchar(' ');

        if((temp>=100000)||(lead_zero>5)) lcd_putchar(c[5]);
        else    if(use_space==SPACE_ALLOW) lcd_putchar(' ');

        if((temp>=10000)||(lead_zero>4)) lcd_putchar(c[4]);
        else    if(use_space==SPACE_ALLOW) lcd_putchar(' ');
        
        if((temp>=1000)||(lead_zero>3))  lcd_putchar(c[3]);
        else    if(use_space==SPACE_ALLOW) lcd_putchar(' ');
        
        if((temp>=100)||(lead_zero>2))   lcd_putchar(c[2]);
        else    if(use_space==SPACE_ALLOW) lcd_putchar(' ');
        
        if((temp>=10)||(lead_zero>1))    lcd_putchar(c[1]);
        else    if(use_space==SPACE_ALLOW) lcd_putchar(' ');
        
        if( dec_point )
			lcd_putchar( '.' );

		lcd_putchar(c[0]);
}

/// Set cursor position x y ///
void lcd_gotoxy(unsigned char x, unsigned char y)
{
        lcd_write_command(_base_y[y]+x);
        lcd_x_cnt=x;
        lcd_y_cnt=y;
}

// write the string str located in SRAM to the LCD
/*void lcd_puts(char *str)
{
char k;
while (k=*str++) lcd_putchar(k);
}*/

// write the string str located in FLASH to the LCD
void lcd_putsf(const char *str)
{
char k;
while (k=*str++) lcd_putchar(k);
}

/// Write higher nibble in 8-bit mode on init stage ///
void lcd_init_write(char cmd)
{
LCD_EN_CLR();
LCD_RW_CLR();//write mode
LCD_RS_CLR();//command mode
LCD_DAT_OUT();//set outputs
LCD_DAT_CLR();

delay_us(2);
LCD_DAT_WR = ( ( (uint32_t)(cmd &0xF0) ) << 6 );//out higher nibble
LCD_EN_SET();

delay_us(2);
LCD_EN_CLR();
delay_ms(10);
}

/// Send to command register ///
void lcd_write_command(char cmd)
{
LCD_RS_CLR();//command mode
lcd_send_data(cmd);
}

/// Send to data register ///
void lcd_write_data(char data)
{
LCD_RS_SET();//data mode
lcd_send_data(data);
}

/// Send prepared data ///
void lcd_send_data(char dta)
{
LCD_EN_CLR();
LCD_RW_CLR();//write mode

delay_us(2);
LCD_DAT_OUT();//set outputs
LCD_DAT_CLR();
LCD_DAT_WR = ( ( (uint32_t)(dta >> 4) ) << 6 ); //out higher nibble
LCD_EN_SET();//latch-up

delay_us(2);
LCD_EN_CLR();

LCD_DAT_CLR();
LCD_DAT_WR = ( ( (uint32_t)(dta & 0x0F) ) << 6 );//out lower nibble

LCD_EN_SET();
delay_us(2);
LCD_EN_CLR();

//LCD_DAT_IN();//set high impendance/deactivate

//delay_ms(10);
while(lcd_read_BF());//wait when LCD will be ready
}

/// Read busy flag ///
char lcd_read_BF(void)
{
LCD_RS_CLR();
return (lcd_read_byte() & 128);//return BF
}

/// Read single byte from LCD port ///
char lcd_read_byte(void)
{
char data = 0;

LCD_EN_CLR();
LCD_RW_SET();//read LCD sygnal
LCD_DAT_IN();//all inputs
delay_us(2);

LCD_EN_SET();//read high nibble
delay_us(2);

data = (LCD_DAT_RD())<<4;
LCD_EN_CLR();
delay_us(2);

LCD_EN_SET();//read low nibble
delay_us(2);

data |= LCD_DAT_RD();

LCD_EN_CLR();
LCD_RW_CLR();

return data;
}

/* function used to define user characters */
/// You should execute lcd_gotoxy() command ///
/// after defining all your chars           ///
void define_char(const unsigned char *pc, unsigned char char_code)
{
unsigned char i,a;
a=(char_code<<3) | 0x40;
for (i=0; i<8; i++)
        {
        //RS=1;
        lcd_write_command(a++);
        lcd_write_data(*pc++);
        };
}

